#ifdef MY_DCF77_POW
/************************************************************************/
/*									                                    */
/*			Decode DCF77 Time Information			                    */
/*									                                    */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/*                                                                      */
/************************************************************************/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "peda_dcf77.h"

//-----------------------------------------------------------------------
#include "pins.h"  // jar, alle Pin #define und Belegungen
#include "s_tools.h"

//-----------------------------------------------------------------------
/*  // ehemals - before
 #define DCF77_IN                            7                // auf mega1284P
 #define DCF77_IN_PORT                       PORTC            // auf mega1284P
 #define DCF77_PIN                           PINC             // auf mega1284P
 #define DCF77_IN_DDR                        DDRC             // auf mega1284P
*/

#define DCF77_PIN       MY_DCF77_IN_PIN
#define DCF77_PORT      MY_DCF77_IN_PORT
#define DCF77_DDR       MY_DCF77_IN_DDR
#define DCF77_IN        MY_DCF77_IN
//-----------------------------------------------------------------------


struct PEDA_TIME newtime;
//struct PEDA_TIME my_time;

//#define DCF_START_ERROR_TRY 3
//#define DCF_START_ERROR_TIME 90

//uint8_t dcf77error_try_start=DCF_START_ERROR_TRY;
uint32_t dcf77_startzeit=0;
uint8_t dcf77error_try=DCF_MAX_ERROR_TRY;

volatile uint8_t dcf77_OK=0;
volatile uint8_t dcf_off_time=0;
volatile uint8_t dcf77error_cnt=0;
volatile uint8_t my_cnt_rdy=0;
volatile uint8_t my_cnt_low=0;


volatile uint8_t my_dcf77_pulse=0;
volatile uint8_t my_dcf77_period=0;
volatile uint8_t merke_my_dcf77_period=0;
uint8_t akt_dcf77_pulse=0;

volatile uint8_t dcf77error = 0;
volatile uint8_t synchronize = 0;				// successful recieved
volatile uint8_t rtc_is_set=0;


uint8_t PROGMEM BITNO[] = {
    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,		// minute
    0xFF,						// parity
    0x10, 0x11, 0x12, 0x13, 0x14, 0x15,			// hour
    0xFF,						// parity
    0x20, 0x21, 0x22, 0x23, 0x24, 0x25,			// day
    0x30, 0x31, 0x32,					// weekday
    0x40, 0x41, 0x42, 0x43, 0x44,			// month
    0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57,	// year
    0xFF };						// parity
uint8_t PROGMEM BMASK[] = { 0x1, 0x2, 0x4, 0x8, 0x10, 0x20, 0x40, 0x80 };

static uint8_t merke_led_status=0;

void dcf_auswertung(void)
{ if(!merke_led_status)
  { merke_led_status=led_test_flags; 
    Serial.print(F("merke: led_status=")); Serial.println(merke_led_status,HEX);
  }
  while(_i2c_busy);
  _i2c_busy=1; RTC.getTime(); _i2c_busy=0;
  sprintf(r_str, "%04d/%02d/%02d_%02d:%02d:%02d", RTC.year, RTC.month, RTC.day, RTC.hour, RTC.minute, RTC.second);

  if( dcf77error_try && (RTC.time2000 >= dcf77_startzeit+((1+DCF_MAX_ERROR_TRY-dcf77error_try)*DCF_MAX_ERROR_TIME)) )
  { Serial.println(r_str);
    Serial.print(F("gewartet:  ")); Serial.print(RTC.time2000-dcf77_startzeit); 
    if( dcf77error_try-2>=(DCF_MAX_ERROR_TRY-(DCF_MAX_ERROR_TRY/2)) )
    { Serial.print(F(" s, DCF 77 reset -> POW off, neuer Versuch Nr. ")); Serial.print(DCF_MAX_ERROR_TRY-(dcf77error_try-2)); 
      Serial.print(F(" von ")); Serial.println(DCF_MAX_ERROR_TRY); 
      dcf_off_time=255;
    } // if( dcf77error_try>(DCF_MAX_ERROR_TRY-(DCF_MAX_ERROR_TRY/2)) )
    else
    { if(dcf77error_try-2)
      { if( led_test_flags&(1<<LED_ON) && 
        ( ((RTC.time2000/3600)%24) >= DCF77_sync_HH && ((RTC.time2000/60)%60) >= DCF77_sync_MM) && 
        ( ((RTC.time2000/3600)%24) <= DCF77_sync_HH+1 && ((RTC.time2000/60)%60) <= DCF77_sync_MM+1) )
        { led_test_flags&=~(1<<LED_ON); FastLED.setBrightness(pgm_read_byte(&pwmtable_16C[0])); FastLED.show(); 
        } // if( led_test_flags&(1<<LED_ON) )

        if( DCF_MAX_ERROR_TRY > (DCF_MAX_ERROR_TRY-(dcf77error_try-2)) )
        { Serial.print(F(" s, DCF 77 reset -> POW off mit LED off, neuer Versuch Nr. ")); Serial.print(DCF_MAX_ERROR_TRY-(dcf77error_try-2)); 
          Serial.print(F(" von ")); Serial.println(DCF_MAX_ERROR_TRY); 
          dcf_off_time=255;
        } // if( DCF_MAX_ERROR_TRY > (DCF_MAX_ERROR_TRY-(dcf77error_try-2)) )
      }       
      else
      { Serial.print(F(" s, DCF 77 reset -> POW off mit LED off, letzter Versuch! von ")); Serial.println(DCF_MAX_ERROR_TRY); 
        dcf_off_time=255;
      } // !( DCF_MAX_ERROR_TRY > (DCF_MAX_ERROR_TRY-(dcf77error_try-2)) )    
    } // !(dcf77error_try-2)
    if(dcf77error_try)
      dcf77error_try--;
  } // if( dcf77error_try && (RTC.time2000 >= dcf77_startzeit+((1+DCF_MAX_ERROR_TRY-dcf77error_try)*DCF_MAX_ERROR_TIME)) )

  if( !dcf77_OK && !rtc_is_set && !dcf77error_try )
  { Serial.println(F(" s ohne Erfolg gewartet")); Serial.println();
    _i2c_busy=1; RTC.getTime(); _i2c_busy=0;

    dcf77_startzeit=RTC.time2000;
    dcf77error_try=DCF_MAX_ERROR_TRY;
    dcf_off_time=0;
    dcf77error_cnt=0;
    my_cnt_rdy=0;
    my_cnt_low=0;

    my_dcf77_pulse=0;
    my_dcf77_period=0;
    merke_my_dcf77_period=0;
    akt_dcf77_pulse=0;

    dcf77error = 0;
    synchronize = 0;				// successful recieved
    rtc_is_set=0;
    
//    dcf77_startzeit=0;
//    MY_DCF77_POW_OFF;                      // power abschalten
//    rtc_is_set=1; dcf77_OK=1; 
#ifdef DCFLED
   leds[DCFLED] = CRGB::Black;
#endif
    led_test_flags=merke_led_status;
    merke_led_status=0;
    FastLED.setBrightness(pgm_read_byte(&pwmtable_16C[led_hell]));
    FastLED.show(); 
    sprintf(r_str, "%02d:%02d Uhr", DCF77_sync_HH, DCF77_sync_MM);
    Serial.print(F("auto DCF77 Sync um ")); Serial.print(r_str); Serial.print(F(" mit max. ")); Serial.print(DCF_MAX_ERROR_TRY); 
    Serial.print(F(" Versuche zu ")); Serial.print((DCF_MAX_ERROR_TIME), DEC); Serial.println(F(" s")); 
    return;  
  } // if( !dcf77_OK && !rtc_is_set && !dcf77error_try )

  if( dcf77_OK && !rtc_is_set )
  { Serial.print(F("\n fehlerfrei, gewartet ")); Serial.print(RTC.time2000-dcf77_startzeit); Serial.print(F(" s, mit >")); Serial.print(DCF_MAX_ERROR_TRY-dcf77error_try); 
    Serial.print(F(" -> Versuche mit max. ")); Serial.print((DCF_MAX_ERROR_TIME/60), DEC); Serial.println(F(" Minuten")); 
    dcf77error_try=0; //rtc_is_set=1; dcf77_startzeit=0;
  
    sprintf(r_str,"%02d:%02d:%02d_%04d/%02d/%02d", RTC.hour, RTC.minute, RTC.second, RTC.year, RTC.month, RTC.day);
    Serial.print(F("RTC old: ")); Serial.println(r_str);
    sprintf(r_str,"%02d:%02d:%02d_%04d/%02d/%02d", bcd2int(newtime.stund), bcd2int(newtime.minut), 0, 2000+bcd2int(newtime.jah), bcd2int(newtime.mona), bcd2int(newtime.tach));
    Serial.print(F("DCF ")); Serial.print(r_str);
#ifdef LED_MY_DCF77_POW
    LED_MY_DCF77_POW_OFF;
#endif // #ifdef LED_MY_DCF77_POW
    if(i2c_test_flags&(1<<I2C_RTC_3231) || i2c_test_flags&(1<<I2C_RTC_1307))
    { if(i2c_test_flags&(1<<I2C_RTC_1307)) { _i2c_busy=1; RTC.stopClock(); _i2c_busy=0; }
      RTC.fillByYMD( 2000+bcd2int(newtime.jah), bcd2int(newtime.mona), bcd2int(newtime.tach) );
      RTC.fillByHMS( bcd2int(newtime.stund), bcd2int(newtime.minut), 0 );
      _i2c_busy=1; RTC.setTime(); _i2c_busy=0;
      if(i2c_test_flags&(1<<I2C_RTC_1307)) { _i2c_busy=1; RTC.startClock(); _i2c_busy=0; }
      (i2c_test_flags&(1<<I2C_RTC_3231)) ? Serial.println(F(" DS3231 RTC ist gestellt")) : Serial.println(F(" DS1307 RTC ist gestellt"));
      _i2c_busy=1; RTC.getTime(); _i2c_busy=0;
      sprintf(r_str, "%04d/%02d/%02d_%02d:%02d:%02d", RTC.year, RTC.month, RTC.day, RTC.hour, RTC.minute, RTC.second);
      Serial.print(F("RTC now: "));Serial.println(r_str);
    } // if(i2c_test_flags&(1<<I2C_RTC_3231) || i2c_test_flags&(1<<I2C_RTC_1307))          
    //Serial.print(F(", Sekunden RTC seit Start = ")); Serial.print(RTC.time2000-dcf77_startzeit);
    led_test_flags=merke_led_status;
    merke_led_status=0;
    leds[DCFLED]  = DCFLED_COL;
leds[WLAN]  = DCFLED_COL;
    if( led_test_flags&(1<<LED_ON) )
    { FastLED.setBrightness(pgm_read_byte(&pwmtable_16C[led_hell]));
      FastLED.show(); 
    }
/*
    MY_DCF77_IN_PORT &= ~(1<<MY_DCF77_IN); // pullup abschalten
    MY_DCF77_IN_DDR  |= (1<<MY_DCF77_IN);  // DDR auf Ausgang
    MY_DCF77_IN_PORT &= ~(1<<MY_DCF77_IN); // Ausgang auf low
    MY_DCF77_POW_OFF;                      // power abschalten
*/
#ifdef LEDOFFSET
    leds[DCFLED] = DCFLED_COL;
    FastLED.show(); 
#endif
    //dcf77_startzeit=0;
    dcf77_startzeit=RTC.time2000;
    dcf77_startzeit=RTC.time2000;
    dcf77error_try=DCF_MAX_ERROR_TRY;
    dcf_off_time=0;
    dcf77error_cnt=0;
    my_cnt_rdy=0;
    my_cnt_low=0;

    my_dcf77_pulse=0;
    my_dcf77_period=0;
    merke_my_dcf77_period=0;
    akt_dcf77_pulse=0;

    dcf77error = 0;
    synchronize = 0;				// successful recieved
    rtc_is_set=0;
    Serial.print(F("DCF Startsekunde RTC = ")); Serial.println(dcf77_startzeit); Serial.println();
    dcf77_OK=0;
    rtc_is_set=0;
    return;  
  } // if( dcf77_OK && !rtc_is_set )
} // void dcf_auswertung(void) //if( dcf77_startzeit )



void dcf_clr(void)
{ dcf77error_cnt++; 
  newtime.sekund = 0;
  newtime.minut = 0;
  newtime.stund = 0;
  newtime.wasday = 0;
  newtime.tach = 0;
  newtime.mona = 0;
  newtime.jah = 0;
}


void decode_dcf77( uint8_t pulse )
{ static uint8_t parity = 0;
  uint8_t i;
  uint8_t *d;

  i = newtime.sekund - 21;
  if( i >= sizeof( BITNO ))			// only bit 21 ... 58
    return;
  parity ^= pulse;				// calculate parity
  i = __LPM_classic__(&BITNO[i]);
  if( i == 0xFF ) // test parity
  { if( parity )
    { dcf77error = 1; dcf_clr(); }
    //  dcf77error = 1;
    parity = 0;
    return;
  } // if( i == 0xFF )
  d = (uint8_t *)&newtime.minut + (i >> 4);	// byte address
  i &= 0x0F;					// bit number
  if( i == 0 )
    *d = 0;					// clear all, if lsb
  if( pulse )
    *d += __LPM_classic__(&BMASK[i]);		// set bit
} // void decode_dcf77( uint8_t pulse )
/*
uint8_t max_p=20;
uint8_t min_p=10;
uint8_t mitt_p=(min_p+((max_p-min_p)/2));;
*/
void scan_dcf77( void )
{ if( my_dcf77_pulse )
  { 
    //Serial.print(my_dcf77_pulse);
/*
    if(my_dcf77_pulse>max_p)
    { max_p=my_dcf77_pulse;
      mitt_p=(min_p+((max_p-min_p)/2));
    }    
    if(my_dcf77_pulse<min_p)
    { min_p=my_dcf77_pulse;
      mitt_p=(min_p+((max_p-min_p)/2));
    }    

if(mitt_p>25) mitt_p=25;
*/
/*
    Serial.print(F(", "));
    Serial.print(max_p);
    Serial.print(F(", "));
    Serial.print(min_p);
    Serial.print(F(", "));
    Serial.println(mitt_p);
*/
//    if( min_p <= my_dcf77_pulse && my_dcf77_pulse < mitt_p )
    if( 5 <= my_dcf77_pulse && my_dcf77_pulse < 13 )
    { decode_dcf77( 0 ); akt_dcf77_pulse=my_dcf77_pulse; }    
    else
    { if( 13 <= my_dcf77_pulse && my_dcf77_pulse <= 22 )
//    { if( mitt_p <= my_dcf77_pulse && my_dcf77_pulse <= max_p )
      { decode_dcf77( 1 ); akt_dcf77_pulse=my_dcf77_pulse; }
      else
      { dcf77error = 1; akt_dcf77_pulse=0; }
    } // ! if( my_dcf77_pulse > 3 && my_dcf77_pulse < 8 )
//    Serial.print("my_dcf77_pulse "); Serial.println(my_dcf77_pulse);
    my_dcf77_pulse = 0;
  } // if( my_dcf77_pulse )

  if( my_dcf77_period )
  { if( newtime.sekund < 60 )
      newtime.sekund++;
    if( 150 < my_dcf77_period && my_dcf77_period < 250 )
    { if( dcf77error == 0 && newtime.sekund == 59 )
      { dcf77_OK=1;
        Serial.println(F("dcf77_OK"));        
        if(!led_test_flags&(1<<LED_ON))
        { led_test_flags|=(1<<LED_ON);       
          Serial.println(F("ws2812_status = LED_ON")); // led_test_flags & (1<<LED_ON);       
        }
      } // if( dcf77error == 0 && newtime.sekund == 59 )
      newtime.sekund = 0;
      dcf77error = 0;
    } // if( dcf77_period > 120 && dcf77_period < 140 )
    else
    { if( my_dcf77_period < 90 || my_dcf77_period > 110 )
      { dcf77error = 1; dcf_clr(); }
    } // ! if( dcf77_period > 120 && dcf77_period < 140 )
    my_dcf77_period = 0;
  } // if( newtime.sekund < 60 )
} // void scan_dcf77( void )

#endif

