#ifndef _PINS_H_
#define _PINS_H_


// ---------------- Name vom Panel---------- Farbe -------------------- Liste ---------------------------------------------------
//                                                            m328p
//                  +5V                      rot              5V        +5V
//                  leer                                                nix
//                  DCF77 LED -              orange/schwarz   D10       DCF77in
//                  DCF77 LED + geschaltet   rot/weiss        D9        LED_DCF77_power
//                  LDR                      orange           A0        LDR blau/weiss
//                  WS2812B                  gelb             D7        WS2812B
//                  TSOP                     violett/weiss    D8        TSOP
//                  GND                      schwarz          GND       GND

#if defined(__AVR_ATmega328P__)
  #define   TIMSKx              TIMSK1
  #define   OCIExA              OCIE1A
  #define   TIMERx_COMPA_vect   TIMER1_COMPA_vect  // ATmega
  #define   TCCRxA              TCCR1A
  #define   COMxA0              COM1A0
  #define   OCRxA               OCR1A
  #define   TCCRxB              TCCR1B
  #define   WGMx2               WGM12
  #define   CSx0                CS10

// arduino nano v3 328p || miniPRO 328p
// DIGITAL
// (Reset)        PC6
// D0 (Rx)        PD0
// D1 (Tx)        PD1
// D2 (Int0)      PD2

#define P_TASTEN
#ifdef P_TASTEN
 // D3 (Int1)      PD3
 #define TASTE_SET          3
 #define TASTE_SET_DDR      DDRD
 #define TASTE_SET_PORT     PORTD
 #define TASTE_SET_DDR_IN   TASTE_SET_DDR|=(1<<TASTE_SET)
 
 // D4 (XCK/T0)    PD4
 #define TASTE_UP           4
 #define TASTE_UP_DDR       DDRD
 #define TASTE_UP_PORT      PORTD
 #define TASTE_UP_DDR_IN    TASTE_UP_DDR|=(1<<TASTE_UP)

 // D5 (T1)        PD5
 #define TASTE_DOWN         5
 #define TASTE_DOWN_DDR     DDRD
 #define TASTE_DOWN_PORT    PORTD
 #define TASTE_DOWN_DDR_IN  TASTE_DOWN_DDR|=(1<<TASTE_DOWN)
#endif

// D6 (AIN0)      PD6														// D6 RTC int ... weiss/schwarz
// DCF77_POW
 #define MY_DCF77_POW          6                                   
#ifdef MY_DCF77_POW
 #define MY_DCF77_POW_DDR      DDRD                                
 #define MY_DCF77_POW_DDR_OUT  MY_DCF77_POW_DDR |=  (1<<MY_DCF77_POW)
 #define MY_DCF77_POW_PORT     PORTD                               
 #define MY_DCF77_POW_ON       MY_DCF77_POW_PORT |=  (1<<MY_DCF77_POW)
 #define MY_DCF77_POW_OFF      MY_DCF77_POW_PORT &= ~(1<<MY_DCF77_POW)
#endif // #ifdef MY_DCF77_POW

// D7 (AIN1)      PD7                                                                                                           
#define STRIP_DATA_PIN 7                                    // D7 WS2812b gelb

// D8 (ICP1)      PB0                                       // TSOP ... Wß-Li                                                                    
// -------------- IR TSOP ----------------
//         violett/weiss      TSOP
#define IRMP_PORT_LETTER      B                             
#define IRMP_BIT_NUMBER       0
#define F_INTERRUPTS 15000

// D9 (OC1A)      PB1 (PWM)                                // D9 LED_DCF77_power     ... rot/weiss
 #define LED_MY_DCF77_POW          1                                   
#ifdef LED_MY_DCF77_POW
 #define LED_MY_DCF77_POW_DDR      DDRB                                
 #define LED_MY_DCF77_POW_DDR_OUT  LED_MY_DCF77_POW_DDR |= (1<<LED_MY_DCF77_POW)
 #define LED_MY_DCF77_POW_PORT     PORTD                               
 #define LED_MY_DCF77_POW_ON       LED_MY_DCF77_POW_PORT|= (1<<LED_MY_DCF77_POW)                             
 #define LED_MY_DCF77_POW_OFF      LED_MY_DCF77_POW_PORT&=~(1<<LED_MY_DCF77_POW)                             
#endif // #ifdef LED_MY_DCF77_POW

// D10(SS/OC1B)   PB2 (PWM)  // D10 DCF77_in        ... 
 #define MY_DCF77_IN             2                
#ifdef MY_DCF77_IN
 #define MY_DCF77_IN_DDR         DDRB
 #define MY_DCF77_IN_DDR_SET_OUT MY_DCF77_IN_DDR|=(1<<MY_DCF77_IN)
 #define MY_DCF77_IN_DDR_SET_IN  MY_DCF77_IN_DDR&=~(1<<MY_DCF77_IN)
 
 #define MY_DCF77_IN_PIN         PINB
 #define MY_DCF77_IN_PORT        PORTB
 #define MY_DCF77_IN_PUP         MY_DCF77_IN_PORT|=(1<<MY_DCF77_IN)

 #define LED_MY_DCF77_ON         MY_DCF77_IN_DDR_SET_OUT;MY_DCF77_IN_PORT&=~(1<<MY_DCF77_IN)
 #define LED_MY_DCF77_OFF        MY_DCF77_IN_DDR_SET_IN;MY_DCF77_IN_PUP
#endif // #ifdef MY_DCF77_IN

// D11(MOSI/OC2)  PB3 (PWM)
// D12(MISO)      PB4

// D13(SCK)       PB5 (LED)
  #define LED_ARDUINO      5													// D13 LED Arduino
  #define LED_ARDUINO_DDR  DDRB
  #define LED_ARDUINO_PORT PORTB
  #define LED_ARDUINO_ON   LED_ARDUINO_PORT|=(1<<LED_ARDUINO)
  #define LED_ARDUINO_OFF  LED_ARDUINO_PORT&=~(1<<LED_ARDUINO)

// ANALOG
// A0 (ADC0)      PC0
#define LICHTSENSOR  0														// A0 LDR blau/weiss

// A1 (ADC1)      PC1														// A1 Taster set braun/weiss
// A2 (ADC2)      PC2														// A2 Taster + gruen/weiss
// A3 (ADC3)      PC3														// A3 Taster - violett/weiss
// A4 (ADC4/SDA)  PC4
// A5 (ADC5/SCL)  PC5
// ----------------- I2C -----------------
#define PIN_SDA   4 //PC4 -> A4													// A4 SDA gelb
#define PIN_SCL   5 //PC5 -> A5													// A5 SCL gruen
/*
#elif defined(__AVR_ATmega1284P__) // --------------------------------------------------------- m1284p -------------------------------------------------------
//                            +---\/---+
//                 (D 0) PB0 1|        |40 PA0 (AI 0 / D24)
 #define DCF77_POW      0                                   // auf mega1284P  D0 DCF77power
 #define DCF77_POW_PORT PORTB                               // auf mega1284P
 #define DCF77_POW_DDR  DDRB                                // auf mega1284P
 #define DCF77_POW_ON    DCF77_POW_PORT |=  (1<<DCF77_POW)
 #define DCF77_POW_OFF   DCF77_POW_PORT &= ~(1<<DCF77_POW)
//                              LDR
 #define                      LICHTSENSOR    0											// A0 LDR blau/weiss 
//                 (D 1) PB1 2|        |39 PA1 (AI 1 / D25)
//            INT2 (D 2) PB2 3|        |38 PA2 (AI 2 / D26)
//             PWM (D 3) PB3 4|        |37 PA3 (AI 3 / D27)
//          PWM/SS (D 4) PB4 5|        |36 PA4 (AI 4 / D28)
//            MOSI (D 5) PB5 6|        |35 PA5 (AI 5 / D29)
//        PWM/MISO (D 6) PB6 7|        |34 PA6 (AI 6 / D30)
 #define IRMP_BIT_NUMBER   6													// D6 TSOP rosa/schwarz
 #define PORT_IRMP         PORTB // auf mega1284P
 #define DDR_IRMP_PORT     DDRB  // auf mega1284P
 #define IRMP_PORT_LETTER  B
//         PWM/SCK (D 7) PB7 8|        |33 PA7 (AI 7 / D31)
 #define LED_ARDUINO       7													// D7 LED Arduino
 #define LED_ARDUINO_PORT  PORTB
 #define LED_ARDUINO_DDR   DDRB
// #define                             RTC_IRQ       7 
//                       RST 9|        |32 AREF
//                      VCC 10|        |31 GND
//                      GND 11|        |30 AVCC
//                    XTAL2 12|        |29 PC7 (D 23)
 #define DCF77_IN                         7                // auf mega1284P
 #define DCF77_IN_PORT                    PORTC            // auf mega1284P
 #define DCF77_PIN                        PINC             // auf mega1284P
 #define DCF77_IN_DDR                     DDRC             // auf mega1284P
//                    XTAL1 13|        |28 PC6 (D 22)

//#define ESP_POWER_PIN  6
//#define ESP_POWER_PORT PORTC
//#define ESP_POWER_DDR  DDRC
//            RX0 (D 8) PD0 14|        |27 PC5 (D 21) TDI
  #define STRIP_DATA_PIN                          21
// #define DATA_PIN                                 21
//                              ws2812
//            TX0 (D 9) PD1 15|        |26 PC4 (D 20) TDO
//      RX1/INT0 (D 10) PD2 16|        |25 PC3 (D 19) TMS
//      TX1/INT1 (D 11) PD3 17|        |24 PC2 (D 18) TCK
 #define                              DCF77_LED   18
//                                I2C
//           PWM (D 12) PD4 18|        |23 PC1 (D 17) SDAglb
 #define                              PIN_SDA     17          //PC1 -> D17
//           PWM (D 13) PD5 19|        |22 PC0 (D 16) SCLgrn
 #define                              PIN_SCL     16          //PC0 -> D16
//           PWM (D 14) PD6 20|        |21 PD7 (D 15) PWM
//                            +--------+
  #define   TIMSKx              TIMSK3
  #define   OCIExA              OCIE3A
  #define   TIMERx_COMPA_vect   TIMER3_COMPA_vect  // ATmega
  #define   TCCRxA              TCCR3A
  #define   COMxA0              COM3A0
  #define   OCRxA               OCR3A
  #define   TCCRxB              TCCR3B
  #define   WGMx2               WGM32
  #define   CSx0                CS30

//#elif defined(__AVR_ATmega1284P__)
  #define LED_ARDUINO_ON       LED_ARDUINO_PORT  |= (1<<LED_ARDUINO)
  #define LED_ARDUINO_OFF      LED_ARDUINO_PORT &= ~(1<<LED_ARDUINO)

#else
 #error "board_wahl_falsch.h"
*/
#endif // defined(__AVR_ATmega1284P__)

#define I2C
  #ifdef I2C
    #define DS1307_ID 0x68
    #define printIIC(args) Wire.write((uint8_t)args)
    #define readIIC() Wire.read()

    #define I2C_TASTATUR       0
    #define I2C_TASTATUR_8574  1
    #define I2C_TASTATUR_8574A 2

//    #define RTC_CHIP           3
    #define RTC_1307           3
    #define RTC_3231           4

    #define EEP_A0             5
    #define EEP_A7             6
    #define EEP_AE             7
  #endif // I2C

#define AUTO_LDR               0
#define LED_ON                 1
#define AUTO_COLOR             2
#define LED_NEXT               3
#define LED_DCF77              4
#define LED_WLAN               5

#define RGB_LED_FB       65280
#define BIG_FB
  
#ifdef  BIG_FB                 // small FB
  #define HELL_UP        92    // #define HELL_UP       5
  #define HELL_DOWN      93    // #define HELL_DOWN     4
  #define NEXT           65    // #define OFF           6
  #define POWER          64    // #define ON            7        

  #define ROT            88    // #define ROT           9
  #define GRUEN          89    // #define GRUEN         8
  #define BLAU           69    // #define BLAU          10
  #define WEISS          68    // #define WEISS         11

  #define ORANGE         84    // #define ORANGE        13
  #define HELL_GRUEN     85    // #define HELL_GRUEN    12
  #define HELL_BLAU      73    // #define HELL_BLAU     14
  #define ROSA1          72    // #define FLASH         15

  #define DUNKEL_ORANGE  80    // #define DUNKEL_ORANGE 21
  #define HELL_CYAN      81    // #define HELL_CYAN     20
  #define DUNKEL_LILA    77    // #define DUNKEL_LILA   22
  #define ROSA2     	 76    // #define STROBE        23

  #define HELL_ORANGE    28    // #define HELL_ORANGE   25
  #define HELL_CYAN      29    // #define CYAN          24
  #define LILA    	 30    // #define PURPUR        26
  #define CYAN1          31    // #define FADE          27

  #define GELB           24    // #define GELB          17
  #define CYAN      	 25    // #define DUNKEL_CYAN   16
  #define PURPUR         26    // #define HELL_LILA     18
  #define CYAN2		 27    // #define SMOOTH         19

  #define ROT_UP	 20    
  #define GRUEN_UP	 21    
  #define BLAU_UP	 22    
  #define QUICK		 23    

  #define ROT_DOWN	 16    
  #define GRUEN_DOWN	 17    
  #define BLAU_DOWN	 18    
  #define SLOW		 19    

  #define DIY1		 12    
  #define DIY2		 13    
  #define DIY3		 14    
  #define AUTO		 15    

  #define DIY4		 8     
  #define DIY5		 9     
  #define DIY6		 10    
  #define FLASH		 11    

  #define JUMP3		 4     
  #define JUMP7   	 5     
  #define FADE3		 6     
  #define FADE7		 7     

#else // small FB              // BIG_FB
  #define HELL_UP        5     // #define HELL_UP        92
  #define HELL_DOWN      4     // #define HELL_DOWN      93
  #define OFF            6     // #define NEXT           65
  #define ON             7     // #define POWER          64

  #define ROT            9     //
  #define GRUEN          8     //
  #define BLAU           10    //
  #define WEISS          11    //

  #define ORANGE         13    //
  #define HELL_GRUEN     12    //
  #define HELL_BLAU      14    //
  #define FLASH          15    //

  #define DUNKEL_ORANGE  21    //
  #define HELL_CYAN      20    //
  #define DUNKEL_LILA    22    //
  #define STROBE         23    //

  #define HELL_ORANGE    25    //
  #define CYAN           24    //
  #define PURPUR         26    //
  #define FADE           27    //

  #define GELB           17    //
  #define DUNKEL_CYAN    16    //
  #define HELL_LILA      18    //
  #define SMOOTH         19    //
#endif

#endif // _PINS_H_ 





